package com.syc.slm.slmbi.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.ObjectUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.syc.slm.common.core.constant.SlmConstants;
import com.syc.slm.slmbi.dao.InstrumentPanelMapper;
import com.syc.slm.slmbi.dto.QueryInstrumentPanelDTO;
import com.syc.slm.slmbi.dto.SaveInstrumentPanelDTO;
import com.syc.slm.slmbi.entity.InstrumentPanel;
import com.syc.slm.slmbi.entity.InstrumentPanelConfig;
import com.syc.slm.slmbi.entity.InstrumentPanelPermission;
import com.syc.slm.slmbi.entity.InstrumentPanelTree;
import com.syc.slm.slmbi.enums.AccessType;
import com.syc.slm.slmbi.enums.NodeType;
import com.syc.slm.slmbi.enums.PublishedStatus;
import com.syc.slm.slmbi.exception.SysException;
import com.syc.slm.slmbi.service.*;
import com.syc.slm.slmbi.vo.InstrumentPanelConfigVo;
import com.syc.slm.slmbi.vo.InstrumentPanelVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Set;

/**
 * slm-bi
 * 仪表板
 *
 * @author : lin
 * @date : 2021-03-25 16:17
 **/
@Slf4j
@Service
public class InstrumentPanelServiceImpl extends ServiceImpl<InstrumentPanelMapper, InstrumentPanel>
    implements InstrumentPanelService {
    @Autowired
    private InstrumentPanelTreeService treeService;

    @Autowired
    private InstrumentPanelPermissionService permissionService;

    @Autowired
    private InstrumentPanelConfigService configService;

    @Autowired
    private InstrumentPanelDraftService draftService;

    @Override
    @Transactional(readOnly = true)
    public IPage<InstrumentPanelVo> selectInstrumentPanelList(String appId, QueryInstrumentPanelDTO where) {
        IPage<InstrumentPanelVo> page = new Page<>(where.getPageNum(), where.getPageSize());
        QueryWrapper<InstrumentPanel> wrapper = new QueryWrapper<>();
        wrapper.eq("p.app_id", appId);
        wrapper.eq("p.group_id", where.getGroupId());
        wrapper.eq("p.record_status", SlmConstants.DATA_VALID);
        if (StringUtils.isNotEmpty(where.getName())) {
            wrapper.like("p.name", where.getName());
        }
        if (StringUtils.isNotEmpty(where.getCreatorName())) {
            wrapper.like("p.creator_name", where.getCreatorName());
        }
        if (StringUtils.isNotEmpty(where.getUpdaterName())) {
            wrapper.like("p.updater_name", where.getUpdaterName());
        }
        if (where.getPublished() != null) {
            wrapper.eq("p.published", where.getPublished());
        }
        if (where.getUpdateTimeBegin() != null) {
            wrapper.gt("p.update_time", where.getUpdateTimeBegin());
        }
        if (where.getUpdateTimeEnd() != null) {
            wrapper.lt("p.update_time", where.getUpdateTimeEnd());
        }
        return baseMapper.selectInstrumentPanelList(page, wrapper);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String savePanel(String appId, SaveInstrumentPanelDTO panel) {
        log.info("保存仪表板");
        return savePanelData(appId, panel);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchPublish(List<String> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw new SysException("请选择你要操作的数据");
        }
        List<InstrumentPanel> panels = baseMapper.selectBatchIds(ids);
        if (CollectionUtils.isNotEmpty(panels)) {
            panels.forEach(x -> {
                x.setPublished(PublishedStatus.PUBLISHED.value);
                baseMapper.updateById(x);
            });
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String delPanel(String id) {
        if (StringUtils.isEmpty(id)) {
            throw new SysException("请选择你要操作的数据");
        }
        InstrumentPanel panel = baseMapper.selectById(id);
        if (SlmConstants.DATA_DELETED.equals(panel.getRecordStatus())) {
            throw new SysException("该数据已被删除，请刷新页面");
        }
        panel.setRecordStatus(SlmConstants.DATA_DELETED);
        baseMapper.updateById(panel);

        log.info("删除该仪表板下的配置");
        delAllSetting(id);
        return id;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String saveAsPanel(String appId, SaveInstrumentPanelDTO panel) {
        log.info("仪表板另存为");
        return savePanelData(appId, panel);
    }

    @Override
    @Transactional(readOnly = true)
    public Boolean showRecoveryBtn(String id) {
        return draftService.getById(id)!=null;
    }

    @Override
    public InstrumentPanelConfigVo recoveryPanel(String id) {
        return configService.getPublishConfigByPanelId(id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String offline(String id) {
        if (StringUtils.isBlank(id)) {
            throw new SysException("请选择你要操作的数据");
        }
        InstrumentPanel panel = baseMapper.selectById(id);
        if (ObjectUtils.isEmpty(panel)) {
            throw new SysException("该数据已不存在，请刷新页面");
        }
        if (SlmConstants.DATA_DELETED.equals(panel.getRecordStatus())) {
            throw new SysException("该数据已被删除，请刷新页面");
        }
        panel.setPublished(PublishedStatus.OFFLINE.value);
        baseMapper.updateById(panel);
        return id;
    }

    @Override
    public void batchUpdatePanelByIds(Set<String> panelIds, Integer recordStatus) {
        if (CollectionUtils.isNotEmpty(panelIds)) {
            baseMapper.batchUpdatePanelByIds(panelIds, recordStatus);
        }
    }

    private String savePanelData(String appId, SaveInstrumentPanelDTO panel) {
        if (StringUtils.isEmpty(panel.getName())) {
            throw new SysException("仪表板名字必填");
        }
        if (StringUtils.isEmpty(panel.getGroupId())) {
            throw new SysException("仪表板分组必填");
        }
        log.info("检查仪表板名称是否重复");
        LambdaQueryWrapper<InstrumentPanel> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(InstrumentPanel::getName, panel.getName());
        wrapper.eq(InstrumentPanel::getAppId, appId);
        if (baseMapper.selectCount(wrapper) > 0) {
            throw new SysException("仪表板名字:" + panel.getName() + "已存在!");
        }

        log.info("保存仪表板访问权限");
        String panelId = IdWorker.get32UUID();
        String accessType = AccessType.PUBLIC.value;
        if (ObjectUtils.isNotEmpty(panel.getPermission()) && StringUtils.isNotEmpty(panel.getPermission()
                                                                                        .getAccessType())) {
            accessType = panel.getPermission().getAccessType();
            if (AccessType.CUSTOM.value.equals(panel.getPermission().getAccessType())) {
                if (CollectionUtils.isNotEmpty(panel.getPermission().getCustomPermissions())) {
                    panel.getPermission().getCustomPermissions().forEach(x -> {
                        InstrumentPanelPermission permission = new InstrumentPanelPermission();
                        permission.setId(IdWorker.get32UUID());
                        permission.setBizId(x.getBizId());
                        permission.setBizName(x.getBizName());
                        permission.setBizType(x.getType());
                        permission.setPanelId(panelId);
                        permission.setPublished(1);
                    });
                }
            }
        }

        InstrumentPanel instrumentPanel = new InstrumentPanel(panelId,
                                                              panel.getName(),
                                                              panel.getGroupId(),
                                                              accessType,
                                                              PublishedStatus.WAIT_PUBLISHED.value);

        log.info("保存仪表板配置");
        if (StringUtils.isNotEmpty(panel.getConfigDetails())) {
            InstrumentPanelConfig config = new InstrumentPanelConfig();
            config.setId(IdWorker.get32UUID());
            config.setConfigDetails(panel.getConfigDetails());
            config.setPanelId(panelId);
            config.setPublished(1);
            configService.save(config);
        }

        log.info("初始化仪表板树");
        treeService.initTree(instrumentPanel.getGroupId(),
                             NodeType.PANEL.value,
                             instrumentPanel.getId(),
                             instrumentPanel.getName());

        log.info("保存仪表板组件树");
        if (CollectionUtils.isNotEmpty(panel.getAssemblys())) {
            InstrumentPanelTree tree = treeService.getById(instrumentPanel.getId());
            for (int i = 0; i < panel.getAssemblys().size(); i++) {
                InstrumentPanelTree t = new InstrumentPanelTree();
                t.setId(tree.getId() + "_" + i);
                t.setNodeType(NodeType.ASSEMBLY.value);
                t.setFullId(tree.getFullId() + "." + t.getId());
                t.setFullName(tree.getFullName() + "." + panel.getAssemblys().get(i).getName());
                t.setAssemblyKey(panel.getAssemblys().get(i).getKey());
                treeService.save(t);
            }
        } else {
            log.info("前端没有构建组件树");
        }
        log.info("保存仪表板数据");
        baseMapper.insert(instrumentPanel);
        return panelId;
    }

    private void delAllSetting(String id) {
        log.info("删除仪表板对应权限");
        permissionService.delPermissionByPanelId(id);

        log.info("删除仪表板对应配置");
        configService.delConfig(id);

        log.info("删除仪表板树");
        treeService.delTreeByPanelId(id);
    }

}
